"""
Plugin name constants for the ArduPilot Methodic Configurator.

This module defines constants for plugin names to maintain DRY principle
and avoid duplication across the codebase.

This file is part of ArduPilot Methodic Configurator. https://github.com/ArduPilot/MethodicConfigurator

SPDX-FileCopyrightText: 2024-2026 Amilcar do Carmo Lucas <amilcar.lucas@iav.de>

SPDX-License-Identifier: GPL-3.0-or-later
"""

# Plugin name constants
PLUGIN_MOTOR_TEST = "motor_test"
PLUGIN_BATTERY_MONITOR = "battery_monitor"
