"""
Auto-generated by the update_configuration_steps_translation.py. Do not edit, ALL CHANGES WILL BE LOST.

This file is part of ArduPilot Methodic Configurator. https://github.com/ArduPilot/MethodicConfigurator

SPDX-FileCopyrightText: 2024-2026 Amilcar Lucas

SPDX-License-Identifier: GPL-3.0-or-later
"""

from ardupilot_methodic_configurator import _

# ruff: noqa: E501
# fmt: off
# pylint: disable=line-too-long


def configuration_steps_strings() -> None:  # noqa: PLR0915 # pylint: disable=too-many-statements
    """
    Translatable strings extracted from all configuration_steps_*.json files.

    For pygettext to extract them, they have no other function
    """
    # why
    _config_steps_strings = _("An even better result for roll and pitch will be achieved by re-autotuning an already (in all axis) autotuned vehicle")
    _config_steps_strings = _("Because the mathematical model of the vehicle allows for analytical PID optimization")
    _config_steps_strings = _("Correct orientation ensures that the flight controller accurately interprets the vehicle's movements and orientation, which is fundamental for stable flight and navigation.")
    _config_steps_strings = _("Ensures the vehicle operates within safe voltage limits and can trigger appropriate failsafe actions to protect the vehicle and its surroundings. It also helps by scaling the PIDs to provide a constant flight behavior, independent of the battery state-of-charge")
    _config_steps_strings = _("Evaluate the aircraft's PID tuning and flight characteristics with feed-forward enable to test faster dynamics")
    _config_steps_strings = _("Evaluating the aircraft's PID tuning and flight characteristics is best done with feed-forward disabled")
    _config_steps_strings = _("For accurate altitude estimation")
    _config_steps_strings = _("For accurate navigation and stabilization at high speeds and/or in windy conditions")
    _config_steps_strings = _("GNSS positioning is required to compensate for the IMU drift. This is required by the configuration and tuning steps. After configuration and tuning are complete the GNSS can be replaced by other positioning system")
    _config_steps_strings = _("Guided mode is used for waypoint navigation and precision flying")
    _config_steps_strings = _("If optical flow is to be used instead of GNSS for positioning")
    _config_steps_strings = _("Optical flow sensor calibration is needed before using it")
    _config_steps_strings = _("Position controller parameters are crucial for waypoint navigation and precision flying")
    _config_steps_strings = _("Precision landing ensures accurate and safe landing and has many parameters")
    _config_steps_strings = _("Propeller size has a big influence on the vehicle dynamics, this adapts the attitude controller response to it")
    _config_steps_strings = _("Proper ESC configuration is crucial for accurate motor control, affecting vehicle performance and safety. It ensures motors respond correctly to control signals and fail safely in various flight conditions.")
    _config_steps_strings = _("Remote controller is mandatory in the initial configuration and tuning phases. Later might be required for operation and/or safety")
    _config_steps_strings = _("Sensors need to be calibrated once, before they can be used")
    _config_steps_strings = _("Some countries require a remote ID for drones to be flown legally.")
    _config_steps_strings = _("Sometimes the weights of the barometer vs. GNSS altitude need to be adjusted.")
    _config_steps_strings = _("System identification is required to create a mathematical model of the vehicle")
    _config_steps_strings = _("Telemetry allows the ground control station to monitor the vehicle's status in real-time, providing flight information for safe and efficient operation.")
    _config_steps_strings = _("The IMU drift is temperature dependent and can cause gyro and/or accel inconsistent errors")
    _config_steps_strings = _("The VTOL-quicktune lua script can safely estimate good PID starting values for the vehicle.")
    _config_steps_strings = _("The compass heading is crucial for the vehicle's navigation and the calibration is very accurate if done during a special flight path.")
    _config_steps_strings = _("The motor thrust linearization is crucial for the throttle controller tuning, and throttle-based notch filter operation")
    _config_steps_strings = _("The notch filter(s) configuration depends on real-flight data.")
    _config_steps_strings = _("The parameter defaults of some parameters are not suitable for the flight tests that will follow")
    _config_steps_strings = _("The throttle controller is crucial for maintaining altitude and controlling the vehicle's vertical movement.")
    _config_steps_strings = _("To ensure the vehicle is safe and stable for routine flying")
    _config_steps_strings = _("To optimize pitch step response.")
    _config_steps_strings = _("To optimize roll step response.")
    _config_steps_strings = _("To optimize yaw D step response.")
    _config_steps_strings = _("To optimize yaw step response.")
    _config_steps_strings = _("To record autotune results after pitch step response optimization")
    _config_steps_strings = _("To record autotune results after roll step response optimization")
    _config_steps_strings = _("To record autotune results after yaw D step response optimization")
    _config_steps_strings = _("To record autotune results after yaw step response optimization")
    _config_steps_strings = _("To record autotune roll-pitch results")
    _config_steps_strings = _("When the gyroscope signal is less noisy the PID gains can be higher without causing motor output oscillations")
    _config_steps_strings = _("With very large or very small vehicles the default PID values are not suitable for the first flight")
    _config_steps_strings = _("parameter values at later steps depend on data that is gathered by means of logging")

    # why_now
    _config_steps_strings = _("After the first flight because it depends on the MOT_THST_HOVER parameter, before the second flight so that it can safely use the altitude controller")
    _config_steps_strings = _("All the previous steps did not require logging, but the following ones do")
    _config_steps_strings = _("Because guided mode requires a tuned position controller")
    _config_steps_strings = _("Because it can only be done after PID tuning is complete")
    _config_steps_strings = _("Because it can only be done after the respective autotune flight")
    _config_steps_strings = _("Because it depends on wind speed estimation")
    _config_steps_strings = _("Because it is optional and should only be done after most other configurations are complete")
    _config_steps_strings = _("Because it needs an initial autotune of all axis")
    _config_steps_strings = _("Because pitch is usually the axis that has the second highest dynamic, and we continue with the second highest dynamic axis")
    _config_steps_strings = _("Because precision landing requires guided mode")
    _config_steps_strings = _("Because roll is usually the axis that has the highest dynamic, and we start with the highest dynamic axis")
    _config_steps_strings = _("Because the position controller PIDs depend on the attitude and attitude-rate PIDs tuned in previous steps")
    _config_steps_strings = _("Because the vehicle should be fully tuned (no more PID changes) before a model gets created (that depends on said PID values)")
    _config_steps_strings = _("Because we first needed system identification flights to create the mathematical model")
    _config_steps_strings = _("Because yaw D can only be done after yaw")
    _config_steps_strings = _("Because yaw is usually the axis that has the third highest dynamic, and we continue with the third highest dynamic axis")
    _config_steps_strings = _("Before the autotune process to estimate if autotune can safely operate the vehicle")
    _config_steps_strings = _("Before the first flight so that it can gather data and safeguard the vehicle from some of the noise")
    _config_steps_strings = _("Before the second flight so that the EKF can be used to estimate the vehicle's position")
    _config_steps_strings = _("Before the second flight so that the vehicle can be safely tuned.")
    _config_steps_strings = _("Can only be done after the connections and device drivers have been configured in the preceding steps")
    _config_steps_strings = _("Done before sensor calibration in Mission Planner to minimize the changes the user has to do in mission planner")
    _config_steps_strings = _("It con only be done after optical flow sensor calibration setup")
    _config_steps_strings = _("It is a pre-requirement for configuring some telemetry systems and ESCs")
    _config_steps_strings = _("Most other parameters are done and these need to be corrected (depending on the vehicle size) before the first flight")
    _config_steps_strings = _("Most parameters have been configured in previous steps. These are parameters that did not fit in any of the categories of the previous steps.")
    _config_steps_strings = _("Needs to be done before compass calibration")
    _config_steps_strings = _("Now that MagFit has been performed the quicktune will work even better")
    _config_steps_strings = _("Pre-requires remote control configuration. Uses GCS telemetry if available. Setting up ESCs early allows for initial motor tests and ensures a stable base for further tuning and flight testing without risking damage to the vehicle.")
    _config_steps_strings = _("Remote ID requires GNSS and air pressure to be configured before, and it must be set up before the first flight")
    _config_steps_strings = _("Requires a basic tune (provided by the previous step) in order to be able to safely navigate in a figure eight flight path.")
    _config_steps_strings = _("Requires the setup (provided by the previous step) and should be done before the second quicktune.")
    _config_steps_strings = _("Requires the setup step. It is easier to do before the FC is mounted in the vehicle")
    _config_steps_strings = _("Sometimes it depends on the remote controller. Configuring telemetry early allows vehicle configuration and sensor calibration in later steps without requiring a USB cable connection to the autopilot")
    _config_steps_strings = _("The failsafe configuration step requires this. Controller PID scaling depends on the battery voltage")
    _config_steps_strings = _("The failsafe configuration step requires this. During flight tests PID scaling depends on the battery voltage")
    _config_steps_strings = _("The following steps require that the orientation is correctly set")
    _config_steps_strings = _("The notch filter setup step requires this, and the safety of the vehicle depends on the ESC and motor configuration")
    _config_steps_strings = _("This is the last step in the tuning process")
    _config_steps_strings = _("We need a good tune before autotune.")
    _config_steps_strings = _("You need to cool down the FC to perform this calibration. It is easier to do before the FC is mounted in the vehicle")
    _config_steps_strings = _("real-flight data is only available after the first flight")

    # blog_text
    _config_steps_strings = _("Adjust the Proportional-Integral-Derivative (PID) controllers based on the vehicle size before the first flight")
    _config_steps_strings = _("Configure Extended Kalman Filter (EKF) noise weights")
    _config_steps_strings = _("Configure logging parameters, including what data is logged and how it's stored")
    _config_steps_strings = _("Configure parameters for everyday use")
    _config_steps_strings = _("Configure parameters for precision landing")
    _config_steps_strings = _("Configure parameters for the first battery, including health monitoring thresholds and failsafe behavior")
    _config_steps_strings = _("Configure parameters for the pitch axis system identification to create a mathematical model of the vehicle")
    _config_steps_strings = _("Configure parameters for the roll axis system identification to create a mathematical model of the vehicle")
    _config_steps_strings = _("Configure parameters for the second battery, including health monitoring thresholds and failsafe behavior")
    _config_steps_strings = _("Configure parameters for the thrust axis system identification to create a mathematical model of the vehicle")
    _config_steps_strings = _("Configure parameters for the yaw axis system identification to create a mathematical model of the vehicle")
    _config_steps_strings = _("Configure parameters for wind speed estimation")
    _config_steps_strings = _("Configure remote controller connection, protocol and settings, including channel mapping and flight modes")
    _config_steps_strings = _("Configure settings for the Electronic Speed Controllers (ESCs), including PWM range and failsafe behavior")
    _config_steps_strings = _("Configure telemetry connection, data protocol and settings between the autopilot and the ground control station(s)")
    _config_steps_strings = _("Configure the GNSS (Global Navigation Satellite System) connection and settings, including antenna phase center location and constellation selection for optimal update rates")
    _config_steps_strings = _("Configure the notch filter settings, used to reduce gyroscope signal noise caused by the motors rotation")
    _config_steps_strings = _("Configure the notch filter(s) based on the data collected from the first flight")
    _config_steps_strings = _("Configure the position controller(s)")
    _config_steps_strings = _("Define the orientation of the flight controller board relative to the vehicle's body frame")
    _config_steps_strings = _("ESC, Motor and propeller configurations")
    _config_steps_strings = _("Evaluate the aircraft's tuning with feed-forward control disabled")
    _config_steps_strings = _("Evaluate the aircraft's tuning with feed-forward control enabled")
    _config_steps_strings = _("General configuration parameters for the vehicle, including flight modes and safety settings")
    _config_steps_strings = _("IMU (Inertial Measurement Unit) temperature calibration results")
    _config_steps_strings = _("IMU (Inertial Measurement Unit) temperature calibration setup")
    _config_steps_strings = _("Initial attitude controller configuration depends on the vehicle's propeller size defined in the component editor window")
    _config_steps_strings = _("Parameters for analytical PID optimization")
    _config_steps_strings = _("Record the results of the in-flight magnetometer calibration")
    _config_steps_strings = _("Record the results of the pitch axis autotuning")
    _config_steps_strings = _("Record the results of the roll and pitch axis retuning, providing data for analysis and adjustment.")
    _config_steps_strings = _("Record the results of the roll axis autotuning, providing data for analysis and adjustment.")
    _config_steps_strings = _("Record the results of the yaw axis autotuning, providing data for analysis and adjustment.")
    _config_steps_strings = _("Record the results of the yaw rate autotuning")
    _config_steps_strings = _("Results of the in-flight VTOL-quicktune lua script PIDs after the MAGFit flight")
    _config_steps_strings = _("Results of the in-flight VTOL-quicktune lua script PIDs before the MAGFit flight.")
    _config_steps_strings = _("Set the remote ID for the vehicle, to comply with local laws if applicable")
    _config_steps_strings = _("Set up mandatory hardware components using Mission Planner")
    _config_steps_strings = _("Set up parameters for barometer compensation")
    _config_steps_strings = _("Set up parameters for guided operation, including waypoint navigation and obstacle avoidance")
    _config_steps_strings = _("Set up parameters for the in-flight magnetometer calibration")
    _config_steps_strings = _("Set up parameters for the pitch axis autotuning process")
    _config_steps_strings = _("Set up parameters for the roll and pitch axis retuning process")
    _config_steps_strings = _("Set up parameters for the roll axis autotuning process")
    _config_steps_strings = _("Set up parameters for the yaw axis autotuning process")
    _config_steps_strings = _("Set up parameters for the yaw rate autotuning process")
    _config_steps_strings = _("Set up the in-flight VTOL-quicktune lua script to tune the PIDs after the MAGFit flight")
    _config_steps_strings = _("Set up the in-flight VTOL-quicktune lua script to tune the PIDs before the MAGFit flight")
    _config_steps_strings = _("Setup optical flow sensor calibration")
    _config_steps_strings = _("Store optical flow sensor calibration results")
    _config_steps_strings = _("Use MOT_THST_HOVER value calculated during the first flight to set throttle controller PIDs")
    _config_steps_strings = _("Use optical flow sensor instead of GNSS sensor for positioning")

    # wiki_text
    _config_steps_strings = _("AutoTune")
    _config_steps_strings = _("Barometer Position Error Compensation")
    _config_steps_strings = _("Battery Monitors (aka Power Monitors/Modules) - BATT2_* parameters")
    _config_steps_strings = _("Battery Monitors (aka Power Monitors/Modules) - BATT_* parameters")
    _config_steps_strings = _("Downloading and Analyzing Data Logs in Mission Planner")
    _config_steps_strings = _("ESC (Electronic Speed Controls)")
    _config_steps_strings = _("Evaluating the aircraft tune")
    _config_steps_strings = _("Extended Kalman filter tuning")
    _config_steps_strings = _("Follow the blog instructions and use Mission Planner instead of this tool to configure the mandatory hardware parameters.")
    _config_steps_strings = _("GPS/Compass (landing page) - GPS* parameters")
    _config_steps_strings = _("Guided Mode")
    _config_steps_strings = _("IAV analytical PID model optimization")
    _config_steps_strings = _("IMU Temperature Calibration")
    _config_steps_strings = _("If lua scripting is not possible do compassmot instead")
    _config_steps_strings = _("If lua scripting is not possible, do a manual tune instead")
    _config_steps_strings = _("Initial parameters calculator")
    _config_steps_strings = _("Managing Gyro Noise with the Dynamic Harmonic Notch Filters")
    _config_steps_strings = _("Manual tuning of Roll and Pitch")
    _config_steps_strings = _("Matlab and Simulink IAV scripts for system identification")
    _config_steps_strings = _("Motor Thrust Scaling")
    _config_steps_strings = _("Mounting the Autopilot - AHRS_ORIENTATION parameter")
    _config_steps_strings = _("Precision Landing and Loiter")
    _config_steps_strings = _("Radio Control Systems")
    _config_steps_strings = _("Remote ID (aka Drone ID)")
    _config_steps_strings = _("Telemetry (landing page)")
    _config_steps_strings = _("Test AltHold")
    _config_steps_strings = _("optical flow sensor setup")
    _config_steps_strings = _("wind speed estimation")

    # external_tool_text
    _config_steps_strings = _("ArduCopter Analytic Tune Tool")
    _config_steps_strings = _("ArduCopter Simulink Model")
    _config_steps_strings = _("ArduPilot Hardware report")
    _config_steps_strings = _("ArduPilot MAGFit in flight compass calibration")
    _config_steps_strings = _("ArduPilot PID Review Tool")
    _config_steps_strings = _("ArduPilot Thrust Expo")
    _config_steps_strings = _("ArduPlane Analytic Tune Tool")
    _config_steps_strings = _("Ardupilot Filter Review tool")
    _config_steps_strings = _("BLHeliSuite32")
    _config_steps_strings = _("Dronetag BS user manual")
    _config_steps_strings = _("EdgeTX Companion")
    _config_steps_strings = _("FlowCal")
    _config_steps_strings = _("Heli Analytic Tune Tool")
    _config_steps_strings = _("Lua script provided by Yuri")
    _config_steps_strings = _("Mission Planner")
    _config_steps_strings = _("Motor/Propeller order and direction test")
    _config_steps_strings = _("New FFT Filter setup and review web tool")
    _config_steps_strings = _("Rover Analytic Tune Tool")
    _config_steps_strings = _("VTOL-quicktune lua script")
    _config_steps_strings = _("free online tool to overlay a grid over an image")
    _config_steps_strings = _("optical flow sensors")
    _config_steps_strings = _("param_pid_adjustment_update.py")

    # jump_messages
    _config_steps_strings = _("""IMU temperature calibration reduces the number of possible 'Accel inconsistent' and 'Gyro inconsistent' errors.
IMU temperature calibration is optional.

Do you want to skip it?""")
    _config_steps_strings = _("""If you are impatient and do not want a fully optimized flight controller
You can skip some steps now.

Jump to '53_everyday_use.param' file?""")
    _config_steps_strings = _("""If you do not use optical flow
you can skip some steps now.

Jump to '53_everyday_use.param' file?""")
    _config_steps_strings = _("""If you do not want an analytical PID optimization
you can skip some steps now.

Jump to '47_position_controller.param' file?""")

    # change_reasons
    _config_steps_strings = _("(Critical voltage + 0.0) x no. of cells")
    _config_steps_strings = _("(Low voltage + 0.0) x no. of cells")
    _config_steps_strings = _("(Max voltage + 0.0) x no. of cells")
    _config_steps_strings = _("Activates the logging of the MAGH.Active message")
    _config_steps_strings = _("Activates the temperature calibration for IMU 1 at the next start")
    _config_steps_strings = _("Activates the temperature calibration for IMU 2 at the next start")
    _config_steps_strings = _("Activates the temperature calibration for IMU 3 at the next start")
    _config_steps_strings = _("Add Navigation logging")
    _config_steps_strings = _("Autotune pitch axis")
    _config_steps_strings = _("Autotune roll and pitch axis")
    _config_steps_strings = _("Autotune roll axis")
    _config_steps_strings = _("Autotune yaw D axis")
    _config_steps_strings = _("Autotune yaw axis")
    _config_steps_strings = _("Defined in component editor")
    _config_steps_strings = _("Derived from vehicle component editor propeller size")
    _config_steps_strings = _("Disable fast harmonic notch logging")
    _config_steps_strings = _("Do not allow arming below this voltage")
    _config_steps_strings = _("ExpressLRS requires FLTMODE_CH != 5")
    _config_steps_strings = _("ExpressLRS requires RC5 to be used for arming (Arm/Disarm function)")
    _config_steps_strings = _("Gather data for the offline IMU temperature calibration while the FC is disarmed")
    _config_steps_strings = _("Gyro raw logging no longer required, notch filter setup is complete, this reduces processor load and log file size")
    _config_steps_strings = _("Hover learn will improve this initial guess")
    _config_steps_strings = _("IMU batch logging no longer required, notch filter setup is complete, this reduces processor load and log file size")
    _config_steps_strings = _("INS_GYRO_FILTER / 2")
    _config_steps_strings = _("Initial value, will be improved at a later step")
    _config_steps_strings = _("Inject chip on the mixer pitch signal")
    _config_steps_strings = _("Inject chip on the mixer roll signal")
    _config_steps_strings = _("Inject chip on the mixer thrust signal")
    _config_steps_strings = _("Inject chip on the mixer yaw signal")
    _config_steps_strings = _("Land ASAP")
    _config_steps_strings = _("Log all but fast att, Nav, Mission, OF, camera, fast IMU, raw, IMU, video stabilization. These are not needed now")
    _config_steps_strings = _("Log disarmed was only required for offline IMU temperature calibration")
    _config_steps_strings = _("Logs measured data both before and after the filters for Filter Review Webtool usage")
    _config_steps_strings = _("No more system identification chip injections")
    _config_steps_strings = _("Normal state for everyday use")
    _config_steps_strings = _("One .bin log file per flight, not per battery/reboot")
    _config_steps_strings = _("Only for IMU and Raw-IMU")
    _config_steps_strings = _("Perform all arming checks. If you have a problem fix its source. Do NOT change this")
    _config_steps_strings = _("Reasonable for most places on this planet")
    _config_steps_strings = _("Reset to default after offline IMU temperature calibration")
    _config_steps_strings = _("Restore value now that system identification is done")
    _config_steps_strings = _("Return and land at home or rally point")
    _config_steps_strings = _("Selected in component editor window")
    _config_steps_strings = _("Selected in the component editor")
    _config_steps_strings = _("So that it can tune the throttle controller on 20_throttle_controller.param file")
    _config_steps_strings = _("Specified in component editor window")
    _config_steps_strings = _("The default is 20Hz but that is too high in most situations")
    _config_steps_strings = _("Total battery capacity specified in the component editor")
    _config_steps_strings = _("Use 1.4 * INS_GYRO_FILTER as a first guess")
    _config_steps_strings = _("Use 2 * MOT_THST_HOVER assuming MOT_THST_HOVER has been correctly learned")
    _config_steps_strings = _("Use INS_GYRO_FILTER / 4 as a first guess")
    _config_steps_strings = _("Use MOT_THST_HOVER assuming MOT_THST_HOVER has been correctly learned")
    _config_steps_strings = _("Use VTOL-Quicktune lua script to estimate a good PID starting values")
    _config_steps_strings = _("Use acc and gyro batch logging on F4 processors or big props, gyro raw logging on others")
    _config_steps_strings = _("Use lua scripting for VTOL-Quicktune, MagFit automation and wind speed estimation automation")
    _config_steps_strings = _("Use pre and post filters acc and gyro batch logging on F4 processors or big props, pre-post gyro raw logging on others")
    _config_steps_strings = _("Use the first notch filter to filter the noise created by the motors/propellers")
    _config_steps_strings = _("Value for the first couple of flights will be changed later once MOT_THST_HOVER is learned")
    _config_steps_strings = _("allow post flight tuning with Replay")
    _config_steps_strings = _("re-enable normal operation, activate input shaping")
    _config_steps_strings = _("test the stabilization loops independent of the input shaping")
    _config_steps_strings = _("was only needed for wind speed estimation")

    # phase_names
    _config_steps_strings = _("Analytical PID optimization")
    _config_steps_strings = _("Assemble all components except the propellers")
    _config_steps_strings = _("Assemble the propellers and perform the first flight")
    _config_steps_strings = _("Basic mandatory configuration")
    _config_steps_strings = _("Everyday use")
    _config_steps_strings = _("Guided operation")
    _config_steps_strings = _("IMU temperature calibration")
    _config_steps_strings = _("Improve altitude control")
    _config_steps_strings = _("Minimalistic mandatory tuning")
    _config_steps_strings = _("Optical flow calibration")
    _config_steps_strings = _("Position controller tuning")
    _config_steps_strings = _("Precision landing")
    _config_steps_strings = _("Standard tuning")

    # phase_descriptions
    _config_steps_strings = _("Assemble all components except the propellers")
    _config_steps_strings = _("Assemble the propellers and perform the first flight")
    _config_steps_strings = _("Calibrate the IMU sensors for at different temperatures")
    _config_steps_strings = _("Guided operation")
    _config_steps_strings = _("Improve altitude under windy conditions")
    _config_steps_strings = _("Improve tuning with more test flight data")
    _config_steps_strings = _("Minimalistic mandatory tuning using test flight data")
    _config_steps_strings = _("Optical flow sensor calibration")
    _config_steps_strings = _("Position controller tuning")
    _config_steps_strings = _("Precision landing")
    _config_steps_strings = _("Set up the basic parameters for the vehicle")
    _config_steps_strings = _("Switch tests off, configure for productive work")
    _config_steps_strings = _("System identification and analytical PID optimization")


# fmt: on


def configuration_steps_descriptions() -> None:
    """
    Translatable strings extracted from the configuration_steps_schema.json file.

    For pygettext to extract them, they have no other function
    """
    _config_steps_descriptions = _("Description of the phase")
    _config_steps_descriptions = _("Explanation of why this step is needed")
    _config_steps_descriptions = _("Explanation of why this step needs to be done at this point")
    _config_steps_descriptions = _("Expression to derive new parameter value")
    _config_steps_descriptions = _("Expression to rename connection")
    _config_steps_descriptions = _("Message about possible jump to another step")
    _config_steps_descriptions = _("Name of tool/process that automatically changes these parameters")
    _config_steps_descriptions = _("Name/description of external tool needed")
    _config_steps_descriptions = _("New value for the parameter")
    _config_steps_descriptions = _("Phases of the configuration process")
    _config_steps_descriptions = _("Previous filenames for this step")
    _config_steps_descriptions = _("Reason for changing the parameter")
    _config_steps_descriptions = _("Reason for the derived parameter")
    _config_steps_descriptions = _("Short description for blog reference")
    _config_steps_descriptions = _("Short description for wiki reference")
    _config_steps_descriptions = _("Starting step number of this phase")
    _config_steps_descriptions = _("Text indicating if step is mandatory or optional with percentages")
    _config_steps_descriptions = _("The name of the plugin to load")
    _config_steps_descriptions = _("URL to blog documentation")
    _config_steps_descriptions = _("URL to external tool")
    _config_steps_descriptions = _("URL to wiki documentation")
    _config_steps_descriptions = _("Where to place the plugin: 'left' for left of scrollable frame, 'top' for above contents")
    _config_steps_descriptions = _("Whether this phase is optional")
